@@ -3,6 +3,7 @@ import click
 import configparser
 import datetime
 import time
+import urllib.request
 import fileinput
 import cwltool.factory
 import cwltool.context
@@ -14,9 +15,10 @@ def cli():
 # Used for autocompletion
 def get_services_list(ctx, args, incomplete):
     # TODO: automate by parsing the docker-compose.yaml
-    return filter(lambda x: x.startswith(incomplete), ['virtuoso', 'graphdb', 'blazegraph', 'comunica',
-    'browse-local-graphdb', 'browse-local-virtuoso', 'browse-local-blazegraph', 'browse-translator-trek',
-    'api', 'drill', 'postgres', 'proxy', 'filebrowser', 'rmljob', 'rmltask'])
+    return filter(lambda x: x.startswith(incomplete), [ 'demo',
+    'graphdb', 'virtuoso', 'tmp-virtuoso', 'blazegraph', 'allegrograph', 'anzograph',
+    'into-the-graph', 'ldf-server', 'comunica', 'notebook',
+    'api', 'drill', 'postgres', 'proxy', 'filebrowser', 'rmljob', 'rmltask' ])
 def get_datasets_list(ctx, args, incomplete):
     return filter(lambda x: x.startswith(incomplete), os.listdir("./datasets"))
 def get_workflows_list(ctx, args, incomplete):
@@ -35,12 +37,14 @@ def get_running_processes(ctx, args, incomplete):
     return [os.system("ps ax | grep -v time | grep '[c]wl-runner' | awk '{print $1}'")]
 
 @cli.command()
+@click.argument('projectname', nargs=1)
 @click.pass_context
-def init(ctx):
-    """Initialize a project in the current directory"""
-    if os.path.exists('.d2sconfig'):
-        click.echo(click.style('[d2s]', bold=True) + ' A ' + click.style('.d2sconfig', bold=True) 
-            + ' file already exists for this directory.', err=True)
+def init(ctx, projectname):
+    """Initialize a project in the provided folder name"""
+    # if os.path.exists('.d2sconfig'):
+    if os.path.exists(projectname):
+        click.echo(click.style('[d2s]', bold=True) + ' The folder ' + click.style(projectname, bold=True) 
+            + ' already exists.', err=True)
         return
 
     config = configparser.ConfigParser()
@@ -53,28 +57,38 @@ def init(ctx):
     click.echo(click.style('[d2s] ', bold=True) + 'Or use the default template repository.')
     d2s_repository_url = click.prompt(click.style('[?]', bold=True) + ' Enter the URL of the d2s git repository to clone in the current directory. Default', default='https://github.com/MaastrichtU-IDS/d2s-transform-template.git')
     config['d2s']['url'] = d2s_repository_url
-    click.echo(click.style('[d2s] ', bold=True) + 'Cloning the repository...')
+    click.echo(click.style('[d2s] ', bold=True) + 'Cloning the repository in ' + projectname + '...')
 
-    os.system('git clone --quiet --recursive ' + d2s_repository_url + ' .')
+    os.system('git clone --quiet --recursive ' + d2s_repository_url + ' ' + projectname)
+    os.chdir(projectname)
     click.echo(click.style('[d2s]', bold=True) + ' Git repository cloned.')
     if not os.path.exists('./d2s-cwl-workflows'):
         os.system('git submodule add --recursive https://github.com/MaastrichtU-IDS/d2s-cwl-workflows.git')
 
     # Create workspace directories
     os.system('mkdir -p workspace/output/tmp-outdir')
+    os.system('mkdir -p workspace/graphdb-import')
+    os.system('chmod -R 777 workspace/graphdb-import')
     os.system('mkdir -p workspace/workflow-history')
     os.system('mkdir -p workspace/download/releases/1')
+    click.echo(click.style('[d2s]', bold=True) + ' Downloading RMLStreamer.jar... [80M]')
+    urllib.request.urlretrieve ("https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar", "workspace/RMLStreamer.jar")
+    # os.system('wget -a workspace/RMLStreamer.jar https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar')
+    os.system('chmod +x workspace/RMLStreamer.jar')
 
     # Copy load.sh in workspace for Virtuoso bulk load
     os.system('mkdir -p workspace/virtuoso && cp d2s-cwl-workflows/support/virtuoso/load.sh workspace/virtuoso')
+    os.system('mkdir -p workspace/tmp-virtuoso && cp d2s-cwl-workflows/support/virtuoso/load.sh workspace/tmp-virtuoso')
+    # TODO: improve this to include it in Docker deployment
 
     click.echo()
     # Copy GraphDB zip file to the right folder in d2s-cwl-workflows
-    click.echo(click.style('[d2s]', bold=True) + ' For the next step you should have ' 
-        + click.style('downloaded GraphDB version 9.1.1 standalone zip', bold=True) 
-        + ' distribution at ' 
-        + click.style('https://ontotext.com/products/graphdb/', bold=True))
-    graphdb_path = click.prompt(click.style('[?]', bold=True) + ' Enter the path to the GraphDB distribution 9.1.1 zip file used to build its image. Default', default='~/graphdb-free-9.1.1-dist.zip')
+    click.echo(click.style('[d2s]', bold=True) + ' The GraphDB triplestore needs to be downloaded for licensing reason.\n'
+        + 'Go to ' 
+        + click.style('https://ontotext.com/products/graphdb/', bold=True) 
+        + ' and provide your email to receive the URL to download ' + click.style('GraphDB version 9.1.1 standalone zip', bold=True))
+    
+    graphdb_path = click.prompt(click.style('[?]', bold=True) + ' Enter the path to the GraphDB distribution 9.1.1 zip file used to build the Docker image. Default', default='~/graphdb-free-9.1.1-dist.zip')
     os.system('cp ' + graphdb_path + ' ./d2s-cwl-workflows/support/graphdb')
     
     with open('.d2sconfig', 'w') as configfile:
@@ -84,8 +98,12 @@ def init(ctx):
     click.echo(click.style('[d2s]', bold=True) + ' Your d2s project has been created!')
     click.echo(click.style('[d2s]', bold=True) + ' The project configuration is stored in the ' 
         + click.style('.d2sconfig', bold=True) + ' file')
-    click.echo(click.style('[d2s]', bold=True) + ' Running ' + click.style('d2s update', bold=True) + '...')
-    ctx.invoke(update)
+    click.secho('cd ' + projectname, bold=True)
+    click.secho('d2s start demo', bold=True)
+    # click.secho('d2s update', bold=True)
+    # Execute update directly:
+    # click.echo(click.style('[d2s]', bold=True) + ' Running ' + click.style('d2s update', bold=True) + '...')
+    # ctx.invoke(update)
 
 
 @cli.command()
@@ -94,8 +112,8 @@ def update():
     os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml pull')
     os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml build graphdb')
     click.echo(click.style('[d2s]', bold=True) + ' All images pulled and built.')
-    click.echo(click.style('[d2s]', bold=True) + ' You can now start services (e.g. drill, and triplestore virtuoso and graphdb):')
-    click.secho('d2s start virtuoso graphdb drill', bold=True)
+    click.echo(click.style('[d2s]', bold=True) + ' You can now start services (e.g. demo services):')
+    click.secho('d2s start demo', bold=True)
 
 
 @cli.command()
@@ -121,26 +139,29 @@ def config():
     help='Use custom deployment config')
 def start(services, deploy):
     """Start services"""
-    services_string = " ".join(services)
-    if deploy == 'trek':
-        print('trek')
-        print('docker-compose -f d2s-cwl-workflows/docker-compose.yaml -f d2s-cwl-workflows/docker-compose.'
-        + deploy + '.yaml up -d --force-recreate ' + services_string)
+    if services[0] == "demo":
+        deploy = "demo"
+        services_string = "graphdb tmp-virtuoso drill api into-the-graph"
+        click.echo(click.style('[d2s] ', bold=True) + ' Starting the services for the demo: ' + services_string)
+    else:
+        services_string = " ".join(services)
+    
+    # Run docker-compose:
+    if deploy:
         os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml -f d2s-cwl-workflows/docker-compose.'
         + deploy + '.yaml up -d --force-recreate ' + services_string)
     else:
-        print('no trek')
         os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml up -d --force-recreate ' + services_string)
 
-        
+    # Ask user to create the GraphDB test repository
     click.echo(click.style('[d2s] ', bold=True) + services_string + ' started.')
-    if 'graphdb' in services:
+    if 'graphdb' in services or 'demo' in services:
         if click.confirm(click.style('[?]', bold=True) + ' Do you want to create the ' 
-        + click.style('test repository', bold=True) + ' in GraphDB?'):
+        + click.style('demo repository', bold=True) + ' in GraphDB?'):
             click.echo(click.style('[d2s] ', bold=True) 
                 + 'Creating the repository, it should take about 20s.')
             time.sleep(10)
-            os.system('curl -X POST http://localhost:7200/rest/repositories -F "config=@d2s-cwl-workflows/support/graphdb-test-repo-config.ttl" -H "Content-Type: multipart/form-data"')
+            os.system('curl -X POST http://localhost:7200/rest/repositories -F "config=@d2s-cwl-workflows/support/graphdb-repo-config.ttl" -H "Content-Type: multipart/form-data"')
     click.echo()
     click.echo(click.style('[d2s]', bold=True) 
         + ' You can now download data to run a first workflow:')
@@ -219,6 +240,23 @@ def download(datasets):
     click.echo(click.style('[d2s]', bold=True) + ' You can now run the transformation workflow:')
     click.secho('d2s run', bold=True)
 
+@cli.command()
+@click.argument('dataset', autocompletion=get_datasets_list)
+# @click.option(
+#     '-p', '--parallelism', default='8', # Not working properly.
+#     help='Run in parallel, depends on Task Slots availables')
+def rml(dataset):
+    """Run RML Streamer"""
+    click.echo(click.style('[d2s]', bold=True) + ' Execute mappings from ' 
+        + click.style('datasets/' + dataset + '/mapping/rml-mappings.ttl', bold=True))
+    rmlstreamer_cmd = 'docker exec -d d2s-cwl-workflows_rmljob_1 /opt/flink/bin/flink run /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/graphdb-import/rml-output-' + dataset + '.nt'
+    # print(rmlstreamer_cmd)
+    os.system(rmlstreamer_cmd)
+    click.echo(click.style('[d2s]', bold=True) + ' Check the job running at ' 
+        + click.style('http://localhost:8078/#/job/running', bold=True))
+    click.echo(click.style('[d2s]', bold=True) + ' Output file in ' 
+        + click.style("workspace/graphdb-import/rml-output-" + dataset + ".nt", bold=True))
+
 @cli.command()
 @click.argument('workflow', autocompletion=get_workflows_list)
 @click.argument('dataset', autocompletion=get_datasets_list)
@@ -239,13 +277,20 @@ def run(workflow, dataset, get_mappings, detached):
     # TODO: Trying to fix issue where virtuoso bulk load only the first dataset we run
     # It needs restart to work a second time
     click.echo(click.style('[d2s] ', bold=True) 
-        + 'Restart Virtuoso and delete file in ' 
+        + 'Restart tmp Virtuoso and delete file in '
         + click.style('workspace/output', bold=True))
-    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml stop virtuoso')
-    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml up -d --force-recreate virtuoso')
+    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml stop tmp-virtuoso')
+    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml up -d --force-recreate tmp-virtuoso')
+    # TODO: fix this dirty Virtuoso deployment 
+    # Virtuoso unable to handle successive bulk load + permission issues + load.sh in the virtuoso containers
+    # I don't know how, they managed to not put it in the container... They had one job...
+
     # Delete previous output (not archived)
     os.system('rm -r workspace/output/*')
-    os.system('rm -r workspace/virtuoso/*.nq')
+    os.system('rm -r workspace/tmp-virtuoso/*.nq')
+    # Make sure the load.sh script is in the tmp Virtuoso folder
+    os.system('sudo chmod -R 777 workspace/tmp-virtuoso')
+    os.system('mkdir -p workspace/tmp-virtuoso && cp d2s-cwl-workflows/support/virtuoso/load.sh workspace/tmp-virtuoso')
     
     if (detached):
         cwl_command = 'nohup time '
@@ -273,18 +318,18 @@ def run(workflow, dataset, get_mappings, detached):
         click.echo()
         click.echo(click.style('[d2s] ', bold=True) + 'Browse the file generated by the workflow in ' 
             + click.style('workspace/output/' + dataset, bold=True))
-        click.echo(click.style('[d2s] ', bold=True)
-            + 'Access the linked data browser for Virtuoso at '
-            + click.style('http://localhost:8891', bold=True))
-        click.echo(click.style('[d2s] ', bold=True)
-            + 'Access Virtuoso (temp store) at '
-            + click.style('http://localhost:8890', bold=True))
-        click.echo(click.style('[d2s] ', bold=True) 
-            + 'Access the linked data browser for GraphDB at '
-            + click.style('http://localhost:7201', bold=True))
-        click.echo(click.style('[d2s] ', bold=True) 
-            + 'Access GraphDB at '
-            + click.style('http://localhost:7200', bold=True))
+        # click.echo(click.style('[d2s] ', bold=True)
+        #     + 'Access the linked data browser for Virtuoso at '
+        #     + click.style('http://localhost:8891', bold=True))
+        # click.echo(click.style('[d2s] ', bold=True)
+        #     + 'Access Virtuoso (temp store) at '
+        #     + click.style('http://localhost:8890', bold=True))
+        # click.echo(click.style('[d2s] ', bold=True) 
+        #     + 'Access the linked data browser for GraphDB at '
+        #     + click.style('http://localhost:7201', bold=True))
+        # click.echo(click.style('[d2s] ', bold=True) 
+        #     + 'Access GraphDB at '
+        #     + click.style('http://localhost:7200', bold=True))
     if (detached):
         click.echo()
         click.echo(click.style('[d2s] ', bold=True) 
